import React from "react";
import styles from "./control.css.style";
import manageJss from "@microsoft/fast-jss-manager-react";
import { classNames } from "@microsoft/fast-web-utilities";
import { mapCSSInlineStyleToCSSPropertyDictionary, } from "@microsoft/fast-tooling/dist/esm/data-utilities/mapping.mdn-data";
import { CSSRef } from "./control.css-ref";
import { memoize } from "lodash-es";
/**
 * This is currently experimental, any use of the CSS control must include the following
 * imports and register with the DesignSystem
 *
 * import { DesignSystem } from "@microsoft/fast-foundation";
 * import {
 *    fastCheckbox,
 *    fastNumberField,
 *    fastOption,
 *    fastSelect,
 *    fastTextField,
 * } from "@microsoft/fast-components";
 * import {
 *     fastToolingColorPicker,
 * } from "@microsoft/fast-tooling/dist/esm/web-components";
 *
 * DesignSystem.getOrCreate().register(
 *    fastCheckbox(),
 *    fastNumberField(),
 *    fastOption(),
 *    fastSelect(),
 *    fastTextField(),
 *    fastToolingColorPicker({ prefix: "fast-tooling" }),
 * );
 */
/**
 * Custom form control definition for CSS
 */
class CSSControl extends React.Component {
    constructor(props) {
        super(props);
        this.handleMultiplePropertyOnChange = (css) => {
            this.resolveCSS(css);
        };
        this.handleOnChange = (propertyName) => {
            return (value) => {
                this.resolveCSS({
                    [propertyName]: value,
                });
            };
        };
        this.resolveCSS = (css) => {
            const cssDictionary = Object.assign(Object.assign({}, this.memoizeMappedStyleToCSSPropertyDictionary(this.props.value)), css);
            this.props.onChange({
                value: `${Object.keys(cssDictionary)
                    .reduce((previousValue, propertyName) => {
                    if (!cssDictionary[propertyName]) {
                        return previousValue;
                    }
                    return `${previousValue} ${propertyName}: ${cssDictionary[propertyName]};`;
                }, "")
                    .trim()}` || undefined,
            });
        };
        this.memoizeMappedStyleToCSSPropertyDictionary = memoize(mapCSSInlineStyleToCSSPropertyDictionary);
    }
    render() {
        return (React.createElement("div", { className: classNames(this.props.managedClasses.css) }, this.renderCSSProperties()));
    }
    renderCSSProperties() {
        const css = Object.assign({}, this.props.css);
        const renderedCssControls = [];
        if (this.props.cssControls) {
            this.props.cssControls.forEach((cssControl) => {
                cssControl.updateProps({
                    css: Object.assign({}, this.memoizeMappedStyleToCSSPropertyDictionary(this.props.value || "")),
                    value: this.props.value,
                    onChange: this.handleMultiplePropertyOnChange,
                });
                renderedCssControls.push(cssControl.render());
                cssControl.getPropertyNames().forEach((propertyName) => {
                    delete css[propertyName];
                });
            });
        }
        return [
            ...renderedCssControls,
            ...Object.entries(css).map(([cssPropertyName, cssProperty]) => {
                return this.renderCSSProperty(cssProperty, cssPropertyName, this.memoizeMappedStyleToCSSPropertyDictionary(this.props.value)[cssPropertyName] || "");
            }),
        ];
    }
    renderCSSProperty(cssProperty, cssPropertyName, cssPropertyValue) {
        if (!cssProperty || !cssProperty.name || !cssProperty.syntax) {
            return null;
        }
        return (React.createElement("fieldset", { key: cssPropertyName },
            React.createElement("legend", null, cssProperty.name),
            React.createElement(CSSRef, { syntax: cssProperty.syntax, onChange: this.handleOnChange(cssPropertyName), mapsToProperty: cssPropertyName, value: cssPropertyValue, dictionaryId: this.props.dictionaryId, dataLocation: this.props.dataLocation })));
    }
}
export { CSSControl };
export default manageJss(styles)(CSSControl);
